"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.raw = void 0;
const path_1 = __importDefault(require("path"));
const file_loader_1 = __importDefault(require("file-loader"));
const loader_utils_1 = __importDefault(require("loader-utils"));
const mime_1 = __importDefault(require("mime"));
const replace_ext_1 = __importDefault(require("replace-ext"));
const schema_utils_1 = __importDefault(require("schema-utils"));
const sharp_1 = __importDefault(require("sharp"));
const options_json_1 = __importDefault(require("./options.json"));
exports.raw = true;
function default_1(content, sourceMap) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const callback = this.async();
    const options = loader_utils_1.default.getOptions(this);
    const params = this.resourceQuery
        ? loader_utils_1.default.parseQuery(this.resourceQuery)
        : undefined;
    if (options)
        schema_utils_1.default(options_json_1.default, options, {
            name: "Image Resize Loader",
            baseDataPath: "options",
        });
    const size = (_a = params === null || params === void 0 ? void 0 : params.size) !== null && _a !== void 0 ? _a : options === null || options === void 0 ? void 0 : options.size;
    const format = (_c = (_b = params === null || params === void 0 ? void 0 : params.format) !== null && _b !== void 0 ? _b : options === null || options === void 0 ? void 0 : options.format) !== null && _c !== void 0 ? _c : getFormat(this.resourcePath);
    const quality = (_d = params === null || params === void 0 ? void 0 : params.quality) !== null && _d !== void 0 ? _d : options === null || options === void 0 ? void 0 : options.quality;
    const scaleUp = (_f = (_e = params === null || params === void 0 ? void 0 : params.scaleUp) !== null && _e !== void 0 ? _e : options === null || options === void 0 ? void 0 : options.scaleUp) !== null && _f !== void 0 ? _f : false;
    const sharpOptions = (_g = params === null || params === void 0 ? void 0 : params.sharpOptions) !== null && _g !== void 0 ? _g : options === null || options === void 0 ? void 0 : options.sharpOptions;
    const fileLoaderOptions = (_h = params === null || params === void 0 ? void 0 : params.fileLoaderOptions) !== null && _h !== void 0 ? _h : options === null || options === void 0 ? void 0 : options.fileLoaderOptions;
    processImage(content, { size, format, quality, scaleUp, sharpOptions })
        .then((result) => {
        const fileLoaderContext = {
            ...this,
            resourcePath: replaceExtension(this.resourcePath, format),
            query: fileLoaderOptions,
        };
        const fileLoaderResult = file_loader_1.default.call(fileLoaderContext, result, sourceMap);
        callback === null || callback === void 0 ? void 0 : callback(null, fileLoaderResult);
    })
        .catch((e) => {
        throw e;
    });
}
exports.default = default_1;
async function processImage(content, { size, format, quality, scaleUp, sharpOptions, }) {
    var _a, _b;
    const sharpImage = sharp_1.default(Buffer.from(content));
    const { height: imageHeight, width: imageWidth, } = await sharpImage.metadata();
    const normalizedImageHeight = imageHeight !== null && imageHeight !== void 0 ? imageHeight : Number.MAX_VALUE;
    const normalizedImageWidth = imageWidth !== null && imageWidth !== void 0 ? imageWidth : Number.MAX_VALUE;
    const normalizedResultHeight = (_a = size === null || size === void 0 ? void 0 : size.height) !== null && _a !== void 0 ? _a : 0;
    const normalizedResultWidth = (_b = size === null || size === void 0 ? void 0 : size.width) !== null && _b !== void 0 ? _b : 0;
    let resultSharp = sharpImage;
    if (scaleUp ||
        (normalizedResultHeight <= normalizedImageHeight &&
            normalizedResultWidth <= normalizedImageWidth))
        resultSharp = sharpImage.resize({
            ...size,
            ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions.resize,
        });
    if (format)
        resultSharp = resultSharp[format]({ quality, ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions[format] });
    return await resultSharp.toBuffer();
}
function replaceExtension(resourcePath, format) {
    if (!format)
        return resourcePath;
    if (mime_1.default.getType(format) === mime_1.default.getType(path_1.default.extname(resourcePath)))
        return resourcePath;
    return replace_ext_1.default(resourcePath, `.${format}`);
}
function getFormat(resourcePath) {
    const type = mime_1.default.getType(resourcePath);
    switch (type) {
        case "image/jpeg":
            return "jpeg";
        case "image/png":
            return "png";
        case "image/webp":
            return "webp";
        case "image/tiff":
            return "tiff";
    }
}
